from sklearn.metrics import mutual_info_score


def run(annotation, ref_annotation):
    """

    `Mutual Information readthedocs
    <https://checkatlas.readthedocs.io/en/latest/metrics/cellannotation/mutual_information/>`__


    :param annotation:
    :param ref_annotation:
    :return:
    """

    return mutual_info_score(annotation, ref_annotation)



from sklearn.metrics import mean_poisson_deviance


def run(annotation, ref_annotation):
    """
    DENOISING

    `Poisson readthedocs
    <https://checkatlas.readthedocs.io/en/latest/metrics/denoising/poisson/>`__

    :param annotation:
    :param ref_annotation:
    :return:
    """
    return mean_poisson_deviance(ref_annotation, annotation)



def run(annotation, ref_annotation):
    """
    `Dunn readthedocs
    <https://checkatlas.readthedocs.io/en/latest/metrics/clustering/dunn/>`__

    TO BE IMPLEMENTED

    :param annotation:
    :param ref_annotation:
    :return:
    """
    return 0



from . import (
    adj_mutual_info,
    adj_rand_index,
    dunn_index,
    fowlkes_mallows,
    isolated_f1_score,
    mutual_info,
    normalized_mutual_info,
    rand_index,
    vmeasure,
)

__all__ = [
    "rand_index",
    "fowlkes_mallow",
    "adj_mutual_info",
    "vmeasure",
    "dunn_index",
    "adj_rand_index",
    "normalized_mutual_info",
    "mutual_info",
    "isolated_f1_score",
]



from sklearn.metrics import f1_score


def run(annotation, ref_annotation):
    """

    `Isolated f1 score readthedocs
    <https://checkatlas.readthedocs.io/en/latest/metrics/cellannotation/f1_score/>`__


    :param annotation:
    :param ref_annotation:
    :return:
    """

    return f1_score(annotation, ref_annotation, average="weighted")



from sklearn.metrics import silhouette_score


def run(count_repr, annotations):
    """
    `Davies-Bouldin readthedocs
    <https://checkatlas.readthedocs.io/en/latest/metrics/clustering/silhouette/>`__

    :param count_repr:
    :param annotations:
    :return:
    """
    return silhouette_score(count_repr, annotations)



from sklearn.metrics import mean_squared_error


def run(annotation, ref_annotation):
    """
    DENOISING

    `Mean Squared Error score readthedocs
    <https://checkatlas.readthedocs.io/en/latest/metrics/denoising/mse/>`__


    :param annotation:
    :param ref_annotation:
    :return:
    """
    return mean_squared_error(annotation, ref_annotation)



import numpy as np
from sklearn.neighbors import NearestNeighbors
import logging

logger = logging.getLogger("checkatlas")


def run(high_dim_counts, low_dim_counts):
    """
    `Entourage readthedocs
    <https://checkatlas.readthedocs.io/en/latest/metrics/dimred/entourage/>`__

    :param high_dim_counts:
    :param low_dim_counts:
    :return:
    """
    # for i in range(4):
    k_neighbors = 4

    # high_dim_counts = high_dim_counts[0:100,]
    # low_dim_counts = low_dim_counts[0:100,]

    X = high_dim_counts
    neigh = NearestNeighbors(n_neighbors=k_neighbors)
    neigh.fit(X)
    A = neigh.kneighbors_graph(X, mode="distance")

    X_low = low_dim_counts
    neigh_low = NearestNeighbors(n_neighbors=k_neighbors)
    neigh_low.fit(X_low)
    B = neigh_low.kneighbors_graph(X_low, mode="distance")

    n = X.shape[0]
    k = k_neighbors

    total_inter = 0
    for i in range(n):
        if i % 100 == 0:
            print(f"i{i} n{n}")
        indice_high = A[i].indices.tolist()
        indice_low = B[i].indices.tolist()

        new_ind_high = np.array(indice_high[1:])
        new_ind_low = np.array(indice_low[1:])

        inter = np.intersect1d(new_ind_high, new_ind_low)
        card = len(inter)
        total_inter = total_inter + card

    entourage_score = total_inter / (n * k)
    logger.debug(f"Entourage : {entourage_score}")

    return entourage_score



from sklearn.metrics import fowlkes_mallows_score


def run(annotation, ref_annotation):
    """

    `Fowlkes-Mallows readthedocs
    <https://checkatlas.readthedocs.io/en/latest/metrics/cellannotation/fowlkes_mallows/>`__

    :param annotation:
    :param ref_annotation:
    :return:
    """
    return fowlkes_mallows_score(annotation, ref_annotation)



from sklearn.metrics.cluster import adjusted_rand_score


def run(annotation, ref_annotation):
    """

    `ARI readthedocs
    <https://checkatlas.readthedocs.io/en/latest/metrics/cellannotation/adjusted_rand_index/>`__


    :param annotation:
    :param ref_annotation:
    :return:
    """
    return adjusted_rand_score(annotation, ref_annotation)



from sklearn.metrics import normalized_mutual_info_score


def run(annotation, ref_annotation):
    """

    `NMI readthedocs
    <https://checkatlas.readthedocs.io/en/latest/metrics/cellannotation/nmi/>`__


    :param annotation:
    :param ref_annotation:
    :return:
    """

    return normalized_mutual_info_score(annotation, ref_annotation)



from sklearn.metrics import v_measure_score


def run(annotation, ref_annotation):
    """

    `V-measure readthedocs
    <https://checkatlas.readthedocs.io/en/latest/metrics/cellannotation/v_measure/>`__


    :param annotation:
    :param ref_annotation:
    :return:
    """
    return v_measure_score(annotation, ref_annotation)



from sklearn.metrics import calinski_harabasz_score


def run(count_repr, annotations):
    """
    `Calinski-Harabasz readthedocs
    <https://checkatlas.readthedocs.io/en/latest/metrics/clustering/calinski_harabasz/>`__

    :param count_repr:
    :param annotations:
    :return:
    """
    return calinski_harabasz_score(count_repr, annotations)
